<?php
/**
 * Free/busy access control based on cached ACL information.
 *
 * PHP version 5
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * Free/busy access control based on cached ACL information.
 *
 * Copyright 2004-2010 Klarälvdalens Datakonsult AB
 * Copyright 2011      Kolab Systems AG
 *
 * See the enclosed file COPYING for license information (LGPL). If you did not
 * receive this file, see
 * http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @author   Steffen Hansen <steffen@klaralvdalens-datakonsult.se>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache_Acl_HordeCache
implements Horde_Kolab_FreeBusy_Cache_Acl
{
    /**
     * The cache handler.
     *
     * @var Horde_Cache
     */
    private $_cache;

    /**
     * Constructor.
     *
     * @param Horde_Cache $cache The cache handler.
     */
    public function __construct($cache)
    {
        $this->_cache = $cache;
    }

    /**
     * Which partials need to be merged into the combined information for one
     * owner?
     *
     * @param Horde_Kolab_FreeBusy_Owner $owner The owner of the partials.
     *
     * @return array The list of partials to be combined.
     */
    public function getPartialIds(Horde_Kolab_FreeBusy_Owner $owner)
    {
        $result = $this->_cache->get($this->_getOwnerKey($owner->getPrimaryId()), 0);
        if ($result !== false) {
            return array_values(unserialize($result));
        } else {
            return array();
        }
    }

    /**
     * Delete all data specific to one user from the cache.
     *
     * @param Horde_Kolab_FreeBusy_Owner $owner The owner to be deleted.
     *
     * @return Horde_Kolab_FreeBusy_Cache_File_Xacl The cache.
     */
    public function deleteOwner(Horde_Kolab_FreeBusy_Owner $owner)
    {
        $this->_cache->expire($this->_getOwnerKey($owner->getPrimaryId()));
    }

    /**
     * Purge the ACL information for a partial.
     *
     * @param Horde_Kolab_FreeBusy_Cache_Partial $partial Partial to forget.
     *
     * @return NULL
     */
    public function delete(Horde_Kolab_FreeBusy_Cache_Partial $partial)
    {
        $id = $partial->getId();
        $removals = unserialize($this->_cache->get($this->_getPartialKey($partial), 0));
        foreach ($removals as $remove => $ac) {
            $key = $this->_getOwnerKey($remove);
            $ids = unserialize($this->_cache->get($key, 0));
            $ids = array_diff($ids, array($id));
            $this->_cache->set($key, serialize($ids), 0);
        }
        $partial->delete();
    }

    /**
     * Store the ACL information for a partial.
     *
     * @param Horde_Kolab_FreeBusy_User          $user     The user accessing the system.
     * @param Horde_Kolab_FreeBusy_Cache_Partial $partial  Partial to store.
     * @param Horde_Kolab_FreeBusy_Resource      $resource Resource handler providing
     *                                                     ACL information.
     *
     * @return array The current ACLs.
     */
    public function store(
        Horde_Kolab_FreeBusy_User $user,
        Horde_Kolab_FreeBusy_Cache_Partial $partial,
        Horde_Kolab_FreeBusy_Resource $resource
    ) {
        $acl = $resource->getAcl();

        /** 
         * Only store the acl information by overwriting if the current user has
         * admin rights on the folder and can actually retrieve the full ACL
         * information. Otherwise the ACL should only be appended.
         */
        if (!isset($acl[$user->getPrimaryId()])
            || (strpos($acl[$user->getPrimaryId()], 'a') === false)) {
            $oldacl = array();
        } else {
            $oldacl = unserialize($this->_cache->get($this->_getPartialKey($partial), 0));
        }

        /* Handle relevance */
        switch ($resource->getRelevance()) {
        case 'readers':
            $perm = 'r';
            break;
        case 'nobody':
            $perm = false;
            break;
        case 'admins':
        default:
            $perm = 'a';
        }

        $filename = $partial->getId();
        /* We remove the filename from all users listed in the old ACL first */
        foreach ($oldacl as $user => $ac) {
            $key = $this->_getOwnerKey($user);
            $ids = unserialize($this->_cache->get($key, 0));
            $ids = array_diff($ids, array($filename));
            $this->_cache->set($key, serialize($ids), 0);
        }

        /* Now add the filename for all users with the correct permissions */
        if ($perm !== false ) {
            foreach ($acl as $user => $ac) {
                if (strpos($ac, $perm) !== false) {
                    if (!empty($user)) {
                        $key = $this->_getOwnerKey($user);
                        $ids = unserialize($this->_cache->get($key, 0));
                        $ids[] = $filename;
                        $this->_cache->set($key, serialize($ids), 0);
                    }
                }
            }
        }

        $this->_cache->set($this->_getPartialKey($partial), serialize($acl), 0);
        return $acl;
    }

    /**
     * Retrieve the old ACL settings for this partial.
     *
     * @param Horde_Kolab_FreeBusy_Cache_File_Acl $filecache The cached data.
     *
     * @return array The old ACL settings.
     */
    private function _getOldAcl($filecache)
    {
        try {
            return $filecache->load();
        } catch (Horde_Kolab_FreeBusy_Exception $e) {
            return array();
        }
    }


    /**
     * Generate a key for an owner.
     *
     * @param string Owner ID.
     *
     * @return string The cache key.
     */
    private function _getOwnerKey($owner)
    {
        return __CLASS__ . '::owner::' . $owner;
    }

    /**
     * Generate a key for a partial.
     *
     * @param Horde_Kolab_FreeBusy_Cache_Partial The partial.
     *
     * @return string The cache key.
     */
    private function _getPartialKey($partial)
    {
        return __CLASS__ . '::partial::' . $partial->getId();
    }
}