<?php
/**
 * Handles a cached partial free/busy list.
 *
 * PHP version 5
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */

/**
 * Handles a cached partial free/busy list.
 *
 * Copyright 2010 Klarälvdalens Datakonsult AB
 * Copyright 2011 Kolab Systems AG
 *
 * See the enclosed file COPYING for license information (LGPL). If you did not
 * receive this file, see
 * http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 *
 * @category Kolab
 * @package  Kolab_FreeBusy
 * @author   Gunnar Wrobel <wrobel@pardus.de>
 * @license  http://www.fsf.org/copyleft/lgpl.html LGPL
 * @link     http://pear.horde.org/index.php?package=Kolab_FreeBusy
 */
class Horde_Kolab_FreeBusy_Cache_Partial_HordeCache
implements Horde_Kolab_FreeBusy_Cache_Partial
{
    /**
     * The ID of the partial.
     *
     * @var string
     */
    private $_id;

    /**
     * The cache handler.
     *
     * @var Horde_Cache
     */
    private $_cache;

    /**
     * The owner.
     *
     * @var Horde_Kolab_FreeBusy_Owner
     */
    private $_owner;

    /**
     * Constructor.
     *
     * @param string                     $id    The ID of the partial.
     * @param Horde_Cache                $cache The cache storing the data.
     * @param Horde_Kolab_FreeBusy_Owner $owner The owner.
     */
    public function __construct(
        $id,
        Horde_Cache $cache,
        Horde_Kolab_FreeBusy_Owner $owner
    )
    {
        $this->_id = $id;
        $this->_cache = $cache;
        $this->_owner = $owner->getPrimaryId();
    }

    /**
     * Get a partial ID representing a resource.
     *
     * @return string ID of the partial.
     */
    public function getId()
    {
        return $this->_id;
    }

    /**
     * Load the extended partial free/busy data.
     *
     * @return Horde_iCalendar The extended partial free/busy data.
     */
    public function load()
    {
        $result = unserialize(
            $this->_cache->get($this->_getPartialKey($this->_id), 0)
        );
        if ($result === false) {
            return false;
        }
        return $result[0];
    }

    /**
     * Load the simple partial free/busy data.
     *
     * @return Horde_iCalendar The reduced partial free/busy data.
     */
    public function loadSimple()
    {
        $pvcal = $this->load();
        if ($pvcal instanceOf Horde_iCalendar) {
            $components = $pvcal->getComponents();
            foreach ($components as $component) {
                if ($component->getType() == 'vFreebusy') {
                    $component->_extraParams = array();
                }
            }
        }
        return $pvcal;
    }

    /**
     * Delete this partial free/busy data.
     *
     * @return NULL
     */
    public function delete()
    {
        $this->_cache->expire($this->_getPartialKey($this->_id));
        $owners = $this->_cache->get($this->getOwnersKey(), 0);
        if ($owners !== false) {
            $owners = unserialize($owners);
            $owners[$this->_owner] = array_diff(
                $owners[$this->_owner], array($this->_id)
            );
            $this->_cache->set(
                $this->getOwnersKey(), serialize($owners), 0
            );
        }
    }

    /**
     * Store partial free/busy data.
     *
     * @param mixed $data The data that should be stored.
     *
     * @return NULL
     */
    public function store($data)
    {
        $this->_cache->set(
            $this->_getPartialKey($this->_id),
            serialize(array($data, time(), $this->_owner)),
            0
        );
        $owners = $this->_cache->get($this->getOwnersKey(), 0);
        if ($owners !== false) {
            $owners = unserialize($owners);
            $owners[$this->_owner][] = $this->_id;
        } else {
            $owners = array($this->_owner => array($this->_id));
        }
        $this->_cache->set(
            $this->getOwnersKey(), serialize($owners), 0
        );
    }

    /**
     * Return the last modification date of the cache file.
     *
     * @return int The last modification date.
     */
    public function getMtime()
    {
        $result = unserialize(
            $this->_cache->get($this->_getPartialKey($this->_id), 0)
        );
        if ($result === false) {
            return false;
        }
        return $result[1];
    }

    /**
     * Generate a key for an owner.
     *
     * @param string Owner ID.
     *
     * @return string The cache key.
     */
    static public function getOwnersKey()
    {
        return __CLASS__ . '::owners';
    }

    /**
     * Generate a key for a partial.
     *
     * @param strung $partial The partial ID.
     *
     * @return string The cache key.
     */
    private function _getPartialKey($partial)
    {
        return __CLASS__ . '::partial::' . $partial;
    }
}